@@ -18,7 +18,9 @@ import (
 
 	"blitiri.com.ar/go/chasquid/internal/config"
 	"blitiri.com.ar/go/chasquid/internal/courier"
+	"blitiri.com.ar/go/chasquid/internal/domaininfo"
 	"blitiri.com.ar/go/chasquid/internal/dovecot"
+	"blitiri.com.ar/go/chasquid/internal/localrpc"
 	"blitiri.com.ar/go/chasquid/internal/maillog"
 	"blitiri.com.ar/go/chasquid/internal/normalize"
 	"blitiri.com.ar/go/chasquid/internal/smtpsrv"
@@ -70,8 +72,6 @@ func main() {
 
 	initMailLog(conf.MailLogPath)
 
-	go signalHandler()
-
 	if conf.MonitoringAddress != "" {
 		go launchMonitoringServer(conf)
 	}
@@ -90,34 +90,20 @@ func main() {
 
 	// Load certificates from "certs/<directory>/{fullchain,privkey}.pem".
 	// The structure matches letsencrypt's, to make it easier for that case.
-	log.Infof("Loading certificates")
+	log.Infof("Loading certificates:")
 	for _, info := range mustReadDir("certs/") {
-		if !info.IsDir() {
-			// Skip non-directories.
+		if info.Type().IsRegular() {
+			// Ignore regular files, we only care about directories.
 			continue
 		}
 
 		name := info.Name()
 		dir := filepath.Join("certs/", name)
-		log.Infof("  %s", name)
-
-		certPath := filepath.Join(dir, "fullchain.pem")
-		if _, err := os.Stat(certPath); os.IsNotExist(err) {
-			continue
-		}
-		keyPath := filepath.Join(dir, "privkey.pem")
-		if _, err := os.Stat(keyPath); os.IsNotExist(err) {
-			continue
-		}
-
-		err := s.AddCerts(certPath, keyPath)
-		if err != nil {
-			log.Fatalf("    %v", err)
-		}
+		loadCert(name, dir, s)
 	}
 
 	// Load domains from "domains/".
-	log.Infof("Domain config paths:")
+	log.Infof("Loading domains:")
 	for _, info := range mustReadDir("domains/") {
 		domain, err := normalize.Domain(info.Name())
 		if err != nil {
@@ -132,7 +118,11 @@ func main() {
 	// as a remote domain (for loops, alias resolutions, etc.).
 	s.AddDomain("localhost")
 
-	dinfo := s.InitDomainInfo(conf.DataDir + "/domaininfo")
+	dinfo, err := domaininfo.New(conf.DataDir + "/domaininfo")
+	if err != nil {
+		log.Fatalf("Error opening domain info database: %v", err)
+	}
+	s.SetDomainInfo(dinfo)
 
 	stsCache, err := sts.NewCache(conf.DataDir + "/sts-cache")
 	if err != nil {
@@ -169,6 +159,10 @@ func main() {
 		log.Fatalf("No address to listen on")
 	}
 
+	go localrpc.DefaultServer.ListenAndServe(conf.DataDir + "/localrpc-v1")
+
+	go signalHandler(dinfo, s)
+
 	s.ListenAndServe()
 }
 
@@ -212,7 +206,7 @@ func initMailLog(path string) {
 	}
 }
 
-func signalHandler() {
+func signalHandler(dinfo *domaininfo.DB, srv *smtpsrv.Server) {
 	var err error
 
 	signals := make(chan os.Signal, 1)
@@ -221,6 +215,8 @@ func signalHandler() {
 	for {
 		switch sig := <-signals; sig {
 		case syscall.SIGHUP:
+			log.Infof("Received SIGHUP, reloading")
+
 			// SIGHUP triggers a reopen of the log files. This is used for log
 			// rotation.
 			err = log.Default.Reopen()
@@ -232,6 +228,17 @@ func signalHandler() {
 			if err != nil {
 				log.Fatalf("Error reopening maillog: %v", err)
 			}
+
+			// We don't want to reload the domain info database periodically,
+			// as it can be expensive, and it is not expected that the user
+			// changes this behind chasquid's back.
+			err = dinfo.Reload()
+			if err != nil {
+				log.Fatalf("Error reloading domain info: %v", err)
+			}
+
+			// Also trigger a server reload.
+			srv.Reload()
 		case syscall.SIGTERM, syscall.SIGINT:
 			log.Fatalf("Got signal to exit: %v", sig)
 		default:
@@ -240,25 +247,46 @@ func signalHandler() {
 	}
 }
 
+// Helper to load a single certificate configuration into the server.
+func loadCert(name, dir string, s *smtpsrv.Server) {
+	log.Infof("  %s", name)
+
+	// Ignore directories that don't have both keys.
+	// We warn about this because it can be hard to debug otherwise.
+	certPath := filepath.Join(dir, "fullchain.pem")
+	if _, err := os.Stat(certPath); err != nil {
+		log.Infof("    skipping: %v", err)
+		return
+	}
+	keyPath := filepath.Join(dir, "privkey.pem")
+	if _, err := os.Stat(keyPath); err != nil {
+		log.Infof("    skipping: %v", err)
+		return
+	}
+
+	err := s.AddCerts(certPath, keyPath)
+	if err != nil {
+		log.Fatalf("    %v", err)
+	}
+}
+
 // Helper to load a single domain configuration into the server.
 func loadDomain(name, dir string, s *smtpsrv.Server) {
 	log.Infof("  %s", name)
 	s.AddDomain(name)
 
-	if _, err := os.Stat(dir + "/users"); err == nil {
-		log.Infof("    adding users")
-		udb, err := userdb.Load(dir + "/users")
-		if err != nil {
-			log.Errorf("      error: %v", err)
-		} else {
-			s.AddUserDB(name, udb)
-		}
+	udb, err := userdb.Load(dir + "/users")
+	if os.IsNotExist(err) {
+		// No users file present, that's okay.
+	} else if err != nil {
+		log.Errorf("    users file error: %v", err)
+	} else {
+		s.AddUserDB(name, udb)
 	}
 
-	log.Infof("    adding aliases")
-	err := s.AddAliasesFile(name, dir+"/aliases")
+	err = s.AddAliasesFile(name, dir+"/aliases")
 	if err != nil {
-		log.Errorf("      error: %v", err)
+		log.Errorf("    aliases file error: %v", err)
 	}
 }
 
