@@ -9,16 +9,16 @@ package main
 import (
 	"bytes"
 	"fmt"
-	"net/url"
 	"os"
 	"path/filepath"
+	"sort"
 	"strconv"
 	"strings"
 	"syscall"
 
-	"blitiri.com.ar/go/chasquid/internal/aliases"
 	"blitiri.com.ar/go/chasquid/internal/config"
 	"blitiri.com.ar/go/chasquid/internal/envelope"
+	"blitiri.com.ar/go/chasquid/internal/localrpc"
 	"blitiri.com.ar/go/chasquid/internal/normalize"
 	"blitiri.com.ar/go/chasquid/internal/trace"
 	"blitiri.com.ar/go/chasquid/internal/userdb"
@@ -36,7 +36,6 @@ Usage:
   chasquid-util [options] aliases-resolve <address>
   chasquid-util [options] domaininfo-remove <domain>
   chasquid-util [options] print-config
-  chasquid-util [options] aliases-add <source> <target>
 
 Options:
   -C=<path>, --configdir=<path>  Configuration directory
@@ -79,7 +78,6 @@ func main() {
 		"aliases-resolve":   aliasesResolve,
 		"print-config":      printConfig,
 		"domaininfo-remove": domaininfoRemove,
-		"aliases-add":       aliasesAdd,
 	}
 
 	cmd := args["$1"]
@@ -223,45 +221,26 @@ func aliasesResolve() {
 	if err != nil {
 		Fatalf("Error loading config: %v", err)
 	}
-	_ = os.Chdir(configDir)
 
-	r := aliases.NewResolver(allUsersExist)
-	r.SuffixSep = *conf.SuffixSeparators
-	r.DropChars = *conf.DropCharacters
-
-	domainDirs, err := os.ReadDir("domains/")
-	if err != nil {
-		Fatalf("Error reading domains/ directory: %v", err)
-	}
-	if len(domainDirs) == 0 {
-		Fatalf("No domains found in config")
-	}
-
-	for _, entry := range domainDirs {
-		name := entry.Name()
-		aliasfile := "domains/" + name + "/aliases"
-		r.AddDomain(name)
-		err := r.AddAliasesFile(name, aliasfile)
-		if err == nil {
-			fmt.Printf("%s: loaded %q\n", name, aliasfile)
-		} else if err != nil && os.IsNotExist(err) {
-			fmt.Printf("%s: no aliases file\n", name)
-		} else {
-			fmt.Printf("%s: error loading %q: %v\n", name, aliasfile, err)
-		}
-	}
-
-	tr := trace.New("chasquid-util", "aliasesResolve")
-	defer tr.Finish()
-
-	rcpts, err := r.Resolve(tr, args["$2"])
+	c := localrpc.NewClient(conf.DataDir + "/localrpc-v1")
+	vs, err := c.Call("AliasResolve", "Address", args["$2"])
 	if err != nil {
 		Fatalf("Error resolving: %v", err)
 	}
-	for _, rcpt := range rcpts {
-		fmt.Printf("%v  %s\n", rcpt.Type, rcpt.Addr)
-	}
 
+	// Result is a map of type -> []addresses.
+	// Sort the types for deterministic output.
+	ts := []string{}
+	for t := range vs {
+		ts = append(ts, t)
+	}
+	sort.Strings(ts)
+
+	for _, t := range ts {
+		for _, a := range vs[t] {
+			fmt.Printf("%v  %s\n", t, a)
+		}
+	}
 }
 
 // chasquid-util print-config
@@ -276,20 +255,15 @@ func printConfig() {
 
 // chasquid-util domaininfo-remove <domain>
 func domaininfoRemove() {
-	domain := args["$2"]
-
 	conf, err := config.Load(configDir+"/chasquid.conf", "")
 	if err != nil {
 		Fatalf("Error loading config: %v", err)
 	}
 
-	// File for the corresponding domain.
-	// Note this is making some assumptions about the data layout and
-	// protoio's storage structure, so it will need adjustment if they change.
-	file := conf.DataDir + "/domaininfo/s:" + url.QueryEscape(domain)
-	err = os.Remove(file)
+	c := localrpc.NewClient(conf.DataDir + "/localrpc-v1")
+	_, err = c.Call("DomaininfoClear", "Domain", args["$2"])
 	if err != nil {
-		Fatalf("Error removing file: %v", err)
+		Fatalf("Error removing domaininfo entry: %v", err)
 	}
 }
 
@@ -297,63 +271,6 @@ func allUsersExist(tr *trace.Trace, user, domain string) (bool, error) {
 	return true, nil
 }
 
-// chasquid-util aliases-add <source> <target>
-func aliasesAdd() {
-	source := args["$2"]
-	target := args["$3"]
-
-	user, domain := envelope.Split(source)
-	if domain == "" {
-		Fatalf("Domain required in source address")
-	}
-
-	if target == "" {
-		Fatalf("Target must be present")
-	}
-
-	// Ensure the domain exists.
-	if _, err := os.Stat(filepath.Join(configDir, "domains", domain)); os.IsNotExist(err) {
-		Fatalf("Domain doesn't exist")
-	}
-
-	conf, err := config.Load(configDir+"/chasquid.conf", "")
-	if err != nil {
-		Fatalf("Error loading config: %v", err)
-	}
-	_ = os.Chdir(configDir)
-
-	// Setup alias resolver.
-	r := aliases.NewResolver(allUsersExist)
-	r.SuffixSep = *conf.SuffixSeparators
-	r.DropChars = *conf.DropCharacters
-
-	r.AddDomain(domain)
-	aliasesFilePath := filepath.Join("domains", domain, "aliases")
-	if err := r.AddAliasesFile(domain, aliasesFilePath); err != nil {
-		Fatalf("%s: error loading %q: %v", domain, aliasesFilePath, err)
-	}
-
-	tr := trace.New("chasquid-util", "aliasesAdd")
-	defer tr.Finish()
-
-	// Check for existing entry.
-	if _, ok := r.Exists(tr, source); ok {
-		Fatalf("There's already an entry for %v", source)
-	}
-
-	// Append the new entry.
-	aliasesFile, err := os.OpenFile(aliasesFilePath, os.O_APPEND|os.O_CREATE|os.O_WRONLY, 0644)
-	if err != nil {
-		Fatalf("Couldn't open %s: %v", aliasesFilePath, err)
-	}
-	_, err = fmt.Fprintf(aliasesFile, "%s: %s\n", user, target)
-	if err != nil {
-		Fatalf("Couldn't write to %s: %v", aliasesFilePath, err)
-	}
-	aliasesFile.Close()
-	fmt.Println("Added alias")
-}
-
 // parseArgs parses the command line arguments, and returns a map.
 //
 // Arguments starting with "-" will be parsed as key-value pairs, and
